



if not exist %LIBRARY_LIB%\\gsl.lib       exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %LIBRARY_LIB%\\gslcblas.lib  exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %LIBRARY_BIN%\\gsl.dll       exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
